#!/usr/bin/ruby

require 'uri'
require 'net/http'
require 'bencoding'

module Btrb

class FileInfo
  def initialize(path, length, sum = nil)
    @path = path
    @length = length
    @md5sum = sum
  end
  attr_reader :path, :length, :md5sum
end

class Metainfo
  include Btrb::Bencoding
  def initialize(path)
    @files = []
    @metainfo = nil
    @uri = URI.parse(path)
    Net::HTTP.start(@uri.host, @uri.port) do |http|
      resp = http.get(@uri.path)
      @metainfo = decode(resp.body)[0]
    end
    if single?
      @files << FileInfo.new([name()], length())
    else
      files().each do |f|
        @files << FileInfo.new(f['path'], f['length'], f['md5sum'])
      end
    end
  end
  def announce()
    @metainfo['announce']
  end
  def name()
    info('name')
  end
  def single?()
    files().nil?
  end
  def size()
    @files.size
  end
  def file(i)
    @files[i]
  end
  def piece_length()
    info('piece length')
  end
  def pieces()
    info('pieces')
  end

  private
  def info(key)
    @metainfo['info'][key]
  end
  def length()
    info('length')
  end
  def files()
    info('files')
  end
end

if $0 == __FILE__
  if ARGV.size == 0
    puts 'usage: ruby torrent.rb torrent-url [more url]'
    exit(1)
  end
  ARGV.each do |path|
    t = Metainfo.new(path)
    0.upto(t.size - 1) do |i|
      p t.file(i)
    end
  end
end

end #moduel Btrb
